# 🎹 PianoRoll Synthesizer Demo 가이드

## 📋 개요

이 데모는 Gradio PianoRoll 컴포넌트와 파이썬 기반 신디사이저를 결합한 예시입니다. 피아노롤에서 노트를 편집하면 파이썬에서 실시간으로 오디오를 생성하여 웹 브라우저에서 재생할 수 있습니다.

## 🚀 실행 방법

```bash
cd demo
python demo_with_synthesizer.py
```

## 🎵 기능 설명

### 📌 Basic Demo 탭
- 기본적인 피아노롤 기능을 테스트할 수 있습니다
- 노트 편집, 가사 입력, 템포 조절 등 기본 기능을 확인할 수 있습니다

### 🎛️ Synthesizer Demo 탭
- **신디사이저 기능**: 파이썬에서 실시간 오디오 생성
- **ADSR 엔벨로프**: Attack, Decay, Sustain, Release 매개변수 조절
- **백엔드 오디오**: 생성된 오디오가 프론트엔드로 전달되어 재생

## 🎼 사용 방법

### 1️⃣ 노트 편집
1. **그리기 모드**: 툴바에서 연필 아이콘을 클릭하고 그리드에 노트 그리기
2. **선택 모드**: 화살표 아이콘을 클릭하고 노트 선택/이동/크기 조절
3. **가사 편집**: 노트를 더블클릭하여 가사 입력
4. **삭제**: 지우개 아이콘을 클릭하고 노트 클릭하여 삭제

### 2️⃣ ADSR 설정
- **Attack**: 소리가 시작되어 최대 볼륨에 도달하는 시간
- **Decay**: 최대 볼륨에서 지속 볼륨으로 감소하는 시간
- **Sustain**: 지속되는 볼륨 레벨 (0-1)
- **Release**: 노트가 끝난 후 소리가 사라지는 시간

### 3️⃣ 오디오 생성 및 재생
1. 노트 편집 완료
2. ADSR 매개변수 조절
3. **"🎶 Synthesize Audio"** 버튼 클릭
4. 오디오 생성 완료 후 피아노롤의 재생 버튼으로 재생

## 🔧 기술적 세부사항

### 신디사이저 구현
- **Oscillator**: 사인파 기반
- **Envelope**: ADSR 엔벨로프 적용
- **Format**: 44.1kHz, 16-bit PCM, 모노
- **Encoding**: base64 인코딩된 WAV 데이터

### 데이터 흐름
1. 피아노롤에서 노트 데이터 수집
2. 파이썬에서 각 노트를 사인파로 변환
3. ADSR 엔벨로프 적용
4. 모든 노트 믹싱
5. WAV로 인코딩 후 base64 변환
6. PianoRoll 컴포넌트의 `audio_data`로 전달
7. 프론트엔드에서 백엔드 오디오 재생

### 추가 데이터 예시
- **Curve Data**: 피치 곡선 (비브라토 효과)
- **Segment Data**: 발음 타이밍 정보

## 🎯 이벤트 시스템

### 지원되는 이벤트
- **play**: 재생 버튼 클릭 시
- **pause**: 일시정지 버튼 클릭 시
- **stop**: 정지 버튼 클릭 시
- **input**: 가사 수정 시 (G2P 실행용)
- **change**: 노트/템포 변경 시

### 사용 예시
```python
def handle_play(event_data):
    print("재생 시작:", event_data)
    # 여기서 추가 재생 로직 구현
    return "재생 중..."

piano_roll.play(handle_play, outputs=status_text)
```

## 🛠️ 커스터마이징

### 신디사이저 개선
- 다른 파형 타입 추가 (square, sawtooth, triangle)
- 필터 추가 (lowpass, highpass)
- 리버브, 딜레이 등 이펙트 추가
- 복잡한 신디사이저 알고리즘 (FM, AM 합성)

### 백엔드 데이터 활용
- 실제 음성 합성 엔진 연동
- 오디오 분석 결과 시각화
- 머신러닝 기반 음악 생성

## 📝 참고사항

- 최대 오디오 길이: 10초
- 샘플 레이트: 44.1kHz
- 동시 재생 가능한 노트 수: 무제한
- 지원 브라우저: Chrome, Firefox, Safari (Web Audio API 지원)

## 🐛 문제 해결

### 오디오가 재생되지 않는 경우
1. 브라우저에서 오디오 재생 권한 확인
2. 개발자 도구 콘솔에서 오류 메시지 확인
3. `use_backend_audio=True` 설정 확인

### 성능 이슈
1. 노트 수를 줄여보세요 (20개 이하 권장)
2. 오디오 길이를 짧게 설정해보세요
3. ADSR 매개변수를 조절해보세요